@@ -45,6 +45,7 @@
 #include "ev-document-transition.h"
 #include "ev-document-forms.h"
 #include "ev-selection.h"
+#include "ev-transition-effect.h"
 #include "ev-attachment.h"
 #include "ev-image.h"
 
@@ -123,10 +124,8 @@ static EvLink     *ev_link_from_action      (PdfDocument       *pdf_document,
 static void        pdf_document_search_free (PdfDocumentSearch *search);
 static void        pdf_print_context_free   (PdfPrintContext   *ctx);
 
-G_DEFINE_TYPE_WITH_CODE (PdfDocument, pdf_document, G_TYPE_OBJECT,
-                         {
-				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT,
-							pdf_document_document_iface_init);
+EV_BACKEND_REGISTER_WITH_CODE (PdfDocument, pdf_document,
+			 {
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_SECURITY,
 							pdf_document_security_iface_init);
 				 G_IMPLEMENT_INTERFACE (EV_TYPE_DOCUMENT_THUMBNAILS,
@@ -859,7 +858,10 @@ ev_link_dest_from_dest (PdfDocument *pdf_document,
 			ev_dest = ev_link_dest_new_xyz (dest->page_num - 1,
 							dest->left,
 							height - dest->top,
-							dest->zoom);
+							dest->zoom,
+							dest->change_left,
+							dest->change_top,
+							dest->change_zoom);
 			g_object_unref (poppler_page);
 		}
 			break;
@@ -874,13 +876,15 @@ ev_link_dest_from_dest (PdfDocument *pdf_document,
 								  MAX (0, dest->page_num - 1));
 			poppler_page_get_size (poppler_page, NULL, &height);
 			ev_dest = ev_link_dest_new_fith (dest->page_num - 1,
-							 height - dest->top);
+							 height - dest->top,
+							 dest->change_top);
 			g_object_unref (poppler_page);
 		}
 			break;
 	        case POPPLER_DEST_FITV:
 			ev_dest = ev_link_dest_new_fitv (dest->page_num - 1,
-							 dest->left);
+							 dest->left,
+							 dest->change_left);
 			break;
 	        case POPPLER_DEST_FITR: {
 			PopplerPage *poppler_page;
@@ -1152,8 +1156,8 @@ pdf_document_document_links_iface_init (EvDocumentLinksIface *iface)
 }
 
 static GList *
-pdf_document_images_get_images (EvDocumentImages *document_images,
-				gint              page)
+pdf_document_images_get_image_mapping (EvDocumentImages *document_images,
+				       gint              page)
 {
 	GList *retval = NULL;
 	PdfDocument *pdf_document;
@@ -1172,8 +1176,11 @@ pdf_document_images_get_images (EvDocumentImages *document_images,
 		image_mapping = (PopplerImageMapping *)list->data;
 
 		ev_image_mapping = g_new (EvImageMapping, 1);
-		
+#ifdef HAVE_POPPLER_PAGE_GET_IMAGE
+		ev_image_mapping->image = ev_image_new (page, image_mapping->image_id);
+#else
 		ev_image_mapping->image = ev_image_new_from_pixbuf (image_mapping->image);
+#endif
 		ev_image_mapping->x1 = image_mapping->area.x1;
 		ev_image_mapping->x2 = image_mapping->area.x2;
 		ev_image_mapping->y1 = image_mapping->area.y1;
@@ -1188,10 +1195,39 @@ pdf_document_images_get_images (EvDocumentImages *document_images,
 	return retval;
 }
 
+GdkPixbuf *
+pdf_document_images_get_image (EvDocumentImages *document_images,
+			       EvImage          *image)
+{
+#ifdef HAVE_POPPLER_PAGE_GET_IMAGE
+	PdfDocument     *pdf_document;
+	PopplerPage     *poppler_page;
+	cairo_surface_t *surface;
+	GdkPixbuf       *retval = NULL;
+
+	pdf_document = PDF_DOCUMENT (document_images);
+	poppler_page = poppler_document_get_page (pdf_document->document,
+						  ev_image_get_page (image));
+
+	surface = poppler_page_get_image (poppler_page, ev_image_get_id (image));
+	if (surface) {
+		retval = ev_document_misc_pixbuf_from_surface (surface);
+		cairo_surface_destroy (surface);
+	}
+
+	g_object_unref (poppler_page);
+
+	return retval;
+#else
+	return GDK_PIXBUF (g_object_ref (ev_image_get_pixbuf (image)));
+#endif /* HAVE_POPPLER_PAGE_GET_IMAGE */
+}
+
 static void
 pdf_document_document_images_iface_init (EvDocumentImagesIface *iface)
 {
-	iface->get_images = pdf_document_images_get_images;
+	iface->get_image_mapping = pdf_document_images_get_image_mapping;
+	iface->get_image = pdf_document_images_get_image;
 }
 
 static GdkPixbuf *
@@ -1964,16 +2000,49 @@ pdf_document_get_page_duration (EvDocumentTransition *trans,
 	return duration;
 }
 
+static EvTransitionEffect *
+pdf_document_get_effect (EvDocumentTransition *trans,
+			 gint                  page)
+{
+	PdfDocument            *pdf_document;
+	PopplerPage            *poppler_page;
+	PopplerPageTransition  *page_transition;
+	EvTransitionEffect     *effect;
+
+	pdf_document = PDF_DOCUMENT (trans);
+	poppler_page = poppler_document_get_page (pdf_document->document, page);
+
+	if (!poppler_page)
+		return NULL;
+
+	page_transition = poppler_page_get_transition (poppler_page);
+
+	if (!page_transition) {
+		g_object_unref (poppler_page);
+		return NULL;
+	}
+
+	/* enums in PopplerPageTransition match the EvTransitionEffect ones */
+	effect = ev_transition_effect_new ((EvTransitionEffectType) page_transition->type,
+					   "alignment", page_transition->alignment,
+					   "direction", page_transition->direction,
+					   "duration", page_transition->duration,
+					   "angle", page_transition->angle,
+					   "scale", page_transition->scale,
+					   "rectangular", page_transition->rectangular,
+					   NULL);
+
+	poppler_page_transition_free (page_transition);
+	g_object_unref (poppler_page);
+
+	return effect;
+}
+
 static void
 pdf_document_page_transition_iface_init (EvDocumentTransitionIface *iface)
 {
 	iface->get_page_duration = pdf_document_get_page_duration;
-}
-
-PdfDocument *
-pdf_document_new (void)
-{
-	return PDF_DOCUMENT (g_object_new (PDF_TYPE_DOCUMENT, NULL));
+	iface->get_effect = pdf_document_get_effect;
 }
 
 /* Forms */
