@@ -28,6 +28,7 @@
  */
 
 #include "ev-properties-view.h"
+#include "ev-backends-manager.h"
 #include "ev-document-factory.h"
 
 #include <config.h>
@@ -133,6 +134,8 @@ nautilus_module_initialize (GTypeModule *module)
 	/* set up translation catalog */
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
+	ev_backends_manager_init ();
 }
 
 void
